function [J, grad] = cofiCostFunc(params, Y, R, num_users, num_movies, num_features, lambda)

%   [J, grad] = COFICOSTFUNC(params, Y, R, num_users, num_movies, num_features, lambda) 
%   returns the cost and gradient for the collaborative filtering problem.

%   X (num_movies  x num_features) -> matrix of movie features
%   Theta (num_users  x num_features) -> matrix of user features
%   Y (num_movies x num_users) -> matrix of user ratings of movies
%   R (num_movies x num_users) -> matrix, where R(i, j) = 1 if the i-th movie was rated by the j-th user
%   X_grad (num_movies x num_features)
%           -> matrix containing the partial derivatives w.r.t. to each element of X
%   Theta_grad (num_users x num_features)
%           -> matrix containing the partial derivatives w.r.t. to each element of Theta

% Unfold the X and Theta matrices from params
X = reshape(params(1:num_movies*num_features), num_movies, num_features);
Theta = reshape(params(num_movies*num_features+1:end), num_users, num_features);

% Cost function (without regularization)
XTheta_Y = X*Theta' - Y; % (num_movies x num_users)
J = 1/2*sum(sum((XTheta_Y.^2).*R));

% Gradient (without regularization) 
% Use the checkCostFunction routine to check that the gradient is correct

X_grad = (XTheta_Y.*R)*Theta; % (num_movies x num_features)
Theta_grad = (XTheta_Y.*R)'*X; % (num_users  x num_features)

% With Regularization
J = J + lambda/2*(sum(sum(Theta.*Theta)) + sum(sum(X.*X)));
X_grad = X_grad + lambda*X;
Theta_grad = Theta_grad + lambda*Theta;

% Mise en forme
grad = [X_grad(:); Theta_grad(:)];

end
